/*
 *  ESP_RCWController_Receiver by Michio Ono.
 *  ESP-WROOM-02 RCWController Receiver
 *  CC BY
 *  Ver 1.0
 */
 
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

const char ssid[] = "****";  //  your network SSID (name)
const char pass[] = "****";  // your network password

WiFiUDP udp;
unsigned int localPort = 10000;
const int PACKET_SIZE = 256;
char packetBuffer[PACKET_SIZE];
bool bConnect=false;

//Buttons
unsigned char nBtn[3],pBtn[3];

//Analog Left
unsigned char nAnL[3],pAnL[3];
int anLDiv=4;//Sensitivity
float anLMax=64.0/127.0-1.0;//Max value=127

//Analog Right
unsigned char nAnR[3],pAnR[3];
int anRDiv=4;//Sensitivity
float anRMax=64.0/127.0-1.0;//Max value=127

//acceleration sensor
unsigned char nAcc[4],pAcc[4];
int accDiv=4;//Sensitivity
float accMax=64.0/127.0-1.0;//Max value=127

void setup() {
  Serial.begin(115200);
  while (!Serial) {
    ;// wait for serial port to connect. Needed for Leonardo only
  }

  WiFi.begin(ssid, pass);
  WiFi.mode(WIFI_STA);
  Serial.println("'Connecting to Access Point");
  Serial.print("'");//for IchigoJam
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  IPAddress myIP = WiFi.localIP();
  
  /*
  WiFi.softAP(ssid, pass);
  IPAddress myIP = WiFi.softAPIP();
  Serial.print("'AP IP address: ");
  */
  Serial.println(myIP);
  
  Serial.println("'Starting UDP");
  udp.begin(localPort);
  Serial.print("'Local port: ");
  Serial.println(udp.localPort());

  nBtn[0]=0x80;
  nAnL[0]=0x81;
  nAnR[0]=0x82;
  nAcc[0]=0x83;
  //nGyA[0]=0x84;
  //nGyU[0]=0x85;
  //Audio=0x86;
  
}

void loop() {
  ReadUDP();
  delay(10);
}

/*********************************************
 * ReadUDP
 * 
 * データ受診
 * 
 **********************************************/

void ReadUDP() {
  bool b=false;
  int i,rlen=0;
  char *pb;
  
  if ((rlen = udp.parsePacket())) {

    /*パケット読み込み*/
    udp.read(packetBuffer, (rlen > PACKET_SIZE) ? PACKET_SIZE : rlen);

    pb=packetBuffer;

    //Button
    for(i=1;i<3;i++) {
      nBtn[i]=pb[i-1];
      if(pBtn[i]!=nBtn[i]) {b=true;pBtn[i]=nBtn[i];}
    }
    if(b) for(i=0;i<3;i++) {Serial.write(nBtn[i]);}

    //Analog Left
    pb+=2; b=false;
    for(i=1;i<3;i++) {
      nAnL[i]=(unsigned char)MaxMin(64+anLDiv*(int)(((float)pb[i-1]-128.0)*anLMax/(float)anLDiv));
      if(pAnL[i]!=nAnL[i]) {b=true;pAnL[i]=nAnL[i];}
    }
    if(b) for(i=0;i<3;i++) {Serial.write(nAnL[i]);}
    
    //Analog Right
    pb+=2; b=false;
    for(i=1;i<3;i++) {
      nAnR[i]=(unsigned char)MaxMin(64+anRDiv*(int)(((float)pb[i-1]-128.0)*anRMax/(float)anRDiv));
      if(pAnR[i]!=nAnR[i]) {b=true;pAnR[i]=nAnR[i];}
    }
    if(b) for(i=0;i<3;i++) {Serial.write(nAnR[i]);}
    
    //Accel
    pb+=2; b=false;
    for(i=1;i<4;i++) {
      nAcc[i]=(unsigned char)MaxMin(64+accDiv*(int)(((float)pb[i-1]-128.0)*accMax/(float)accDiv));
      if(pAcc[i]!=nAcc[i]) {b=true;pAcc[i]=nAcc[i];}
    }
    if(b) for(i=0;i<4;i++) {Serial.write(nAcc[i]);}
  }
  
}

/*********************************************
 * 最大値・最小値
 * 
 * 1-127: center 64, from -63 to 63 
 * 
 **********************************************/

inline int max(int a,int b) {return ((a)>(b)?(a):(b)); }
inline int min(int a,int b) {return ((a)<(b)?(a):(b)); }
int MaxMin(int n) {
  return min(max(1,n),127);
}

