/*
 *  ESP_Unipad_Receiver by Michio Ono.
 *  ESP-WROOM-02 Unipad data reciever
 *  OSC Edition
 *  CC BY
 *  Ver 1.1.0
 *   
 *  latest version
 *  http://ijutilities.micutil.com
 * 
 */

/******************************************
 * References
 * ****************************************
 * 
 * Unipad x ESP8266
 * http://unipad.digitalray.co.jp
 * 
 * tested on ESPr Developer
 * https://www.switch-science.com/catalog/2500/
 * 
 * IchigoJam
 * http://ichigojam.net
 * 
 */ 

/*****************************************/
//[how to use]
/*****************************************/
//
// 1. Download Unipad on your smartphone
//  http://unipad.digitalray.co.jp/download/
//
// 2. Launch Unipad. -> new connection -> 
//   name : any you want.
//   ipaddress: none (input nothing)
//   port:6666 (or your config)
//   protcol:OSC
//
//   tap connection!
//
// 3. shown GamePad on your smartphone,
//     when connection established.
//
// 4. check serial console on Arduino IDE.
//

/********************************************
 * OSC library
 * ******************************************
 * this scripts needs  OSC library
 * download and add library
 * https://github.com/CNMAT/OSC
 * 
 *******************************************/
#include <OSCMessage.h>
#include <OSCBundle.h>
#include <OSCData.h>

#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

//----------------------------------start config
const unsigned int outPort = 6666;

int padType = 1;//gamepad

//your Wifi's ssid
const char ssid[] = "****";

//your Wifi's password
const char pass[] = "****";

//----------------------------------end config

WiFiUDP Udp;
IPAddress outIp;
OSCErrorCode error;
bool bConnect=false;
bool b=false;
char d[16];

//Buttons
unsigned char nBtn[3];
unsigned char pBtn[3];

//touchArea & tachArea1
unsigned char nPos[3],pPos[3];
int posDiv=4;//Sensitivity
float posMax=64.0/0.5-1.0;//Max value=0.5

//touchArea2
unsigned char nAnR[3],pAnR[3];
int anRDiv=4;//Sensitivity
float anRMax=64.0/0.5-1.0;//Max value=0.5

//acceleration sensor
unsigned char nAcc[4],pAcc[4];
int accDiv=4;//Sensitivity
float accMax=64.0/0.5-1.0;//Max value=5

//gyroscope attitude (姿勢）
unsigned char nGyA[5],pGyA[5];
int gyaDiv=4;//Sensitivity
float gyaMax=64.0/1.0-1.0;//Max value=1

//gyroscope Unbiased (回転角度）
unsigned char nGyU[4],pGyU[4];
int gyuDiv=4;//Sensitivity
float gyuMax=64.0/4.0-1.0;//Max value=4

//Audio
unsigned char nAudio,pAudio;
int audDiv=4;//Sensitivity
float audMax=128.0/10.0-1.0;//Max value=10


/*********************************************/
void setup() {

  Serial.begin(115200);
  // Connect to WiFi network
  //Serial.println();
  //Serial.print("'Connecting to ");
  //Serial.println(ssid);

  WiFi.begin(ssid, pass);
  //Serial.print("'");
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  //Serial.println("");
  //Serial.println("'WiFi connected");
  
  // Print the IP address
  //Serial.print("'");
  Serial.println(WiFi.localIP());
  Udp.begin(outPort);

  nBtn[0]=0x80;
  nPos[0]=0x81;
  nAnR[0]=0x82;
  nAcc[0]=0x83;
  nGyA[0]=0x84;
  nGyU[0]=0x85;
  //Audio=0x86

}

/*********************************************/
void loop() {

  ReadOSC();
  delay(10);

}

/*********************************************
 * 最大値・最小値
 * 
 * 1-127: center 64, from -63 to 63 
 * 
 **********************************************/

inline int max(int a,int b) {return ((a)>(b)?(a):(b)); }
inline int min(int a,int b) {return ((a)<(b)?(a):(b)); }
int MaxMin(int n) {
  return min(max(1,n),127);
}

/*********************************************
 * doTouchButton
 * 
 * ボタンが押された時の処理 
 * 
 **********************************************/
void doTouchButton(OSCMessage &msg, int rType) {

  char str[8];
  int i = 0;
  int n=msg.size();//number of data

  if(n>0) {
    while ( i < n ) {
      msg.getString(i, str, 8);
      if (strncmp(str, "Up", sizeof(str)) == 0) {
        if(rType<2) {nBtn[2]&=0b11111110; nBtn[2]|=rType;}
      } else if (strncmp(str, "Down", sizeof(str)) == 0) {
        if(rType<2) {nBtn[2]&=0b11111101; nBtn[2]|=(rType<<1);}
      } else if (strncmp(str, "Right", sizeof(str)) == 0) {
        if(rType<2) {nBtn[2]&=0b11111011; nBtn[2]|=(rType<<2);}
      } else if (strncmp(str, "Left", sizeof(str)) == 0) {
        if(rType<2) {nBtn[2]&=0b11110111; nBtn[2]|=(rType<<3);}
      } else if (strncmp(str, "ButtonY", sizeof(str)) == 0) {
        if(rType<2) {nBtn[2]&=0b11101111; nBtn[2]|=(rType<<4);}
      } else if (strncmp(str, "ButtonB", sizeof(str)) == 0) {
        if(rType<2) {nBtn[2]&=0b11011111; nBtn[2]|=(rType<<5);}
      } else if (strncmp(str, "ButtonA", sizeof(str)) == 0) {
        if(rType<2) {nBtn[2]&=0b10111111; nBtn[2]|=(rType<<6);}
        
      } else if (strncmp(str, "ButtonX", sizeof(str)) == 0) {
        if(rType<2) {nBtn[1]&=0b11111110; nBtn[1]|=rType;}
      } else if (strncmp(str, "ButtonL1", sizeof(str)) == 0) {
        if(rType<2) {nBtn[1]&=0b11111101; nBtn[1]|=(rType<<1);}
      } else if (strncmp(str, "ButtonL2", sizeof(str)) == 0) {
        if(rType<2) {nBtn[1]&=0b11111011; nBtn[1]|=(rType<<2);}
      } else if (strncmp(str, "ButtonR1", sizeof(str)) == 0) {
        if(rType<2) {nBtn[1]&=0b11110111; nBtn[1]|=(rType<<3);}
      } else if (strncmp(str, "ButtonR2", sizeof(str)) == 0) {
        if(rType<2) {nBtn[1]&=0b11101111; nBtn[1]|=(rType<<4);}

      } else if (strncmp(str, "ButtonS1", sizeof(str)) == 0) {
        if(rType<2) {nBtn[2]&=0b11111100; nBtn[2]|=(0x03);}
      } else if (strncmp(str, "ButtonS2", sizeof(str)) == 0) {
        if(rType<2) {nBtn[2]&=0b11110011; nBtn[2]|=(0x0C);}

      } else if (strncmp(str, "TouchArea", sizeof(str)) == 0) {
        nPos[1]=nPos[2]=pPos[1]=pPos[2]=64;
        for(i=0;i<3;i++) {Serial.write(nPos[i]);}
        return;
      } else if (strncmp(str, "TouchArea2", sizeof(str)) == 0) {
        nAnR[1]=nAnR[2]=pAnR[1]=pAnR[2]=64;
        for(i=0;i<3;i++) {Serial.write(nAnR[i]);}
        return;

      } else if (strncmp(str, "menu", sizeof(str)) == 0) {
        return;//btn[MENU] = 1;
      }
      i++;
    } //endwhile
  
    //状態を送信
    if( (pBtn[1]!=nBtn[1]) || (pBtn[2]!=nBtn[2]) ) {
      Serial.write(nBtn[0]);Serial.write(nBtn[1]); Serial.write(nBtn[2]);
      pBtn[1]=nBtn[1];pBtn[2]=nBtn[2];
    }
  
  } else {
    //解放状態
    nBtn[1]=pBtn[1]=nBtn[2]=pBtn[2]=0;
    Serial.write(nBtn[0]);Serial.write(nBtn[1]); Serial.write(nBtn[2]);
  }
  
}

void touchEnd(OSCMessage &msg) { doTouchButton(msg,0); }
void touchStart(OSCMessage &msg) { doTouchButton(msg,1); }
void touchRepeat(OSCMessage &msg) { doTouchButton(msg,2); }

void touchPos(OSCMessage &msg) {
  int i; bool b=false;
  for(i=1;i<3;i++) {
    msg.getString(i-1,d,16);
    nPos[i]=(unsigned char)MaxMin(64+posDiv*(int)(atof(d)*posMax/(float)posDiv));
    if(pPos[i]!=nPos[i]) {b=true;pPos[i]=nPos[i];}
  }
  if(b) for(i=0;i<3;i++) {Serial.write(nPos[i]);}
}

void touchPos2(OSCMessage &msg) {
  int i; bool b=false;
  for(i=1;i<3;i++) {
    msg.getString(i-1,d,16);
    nAnR[i]=(unsigned char)MaxMin(64+anRDiv*(int)(atof(d)*anRMax/(float)anRDiv));
    if(pAnR[i]!=nAnR[i]) {b=true;pAnR[i]=nAnR[i];}
  }
  if(b) for(i=0;i<3;i++) {Serial.write(nAnR[i]);}
}

void sensorAccel(OSCMessage &msg) {
  int i; b=false;
  for(i=1;i<4;i++) {
    msg.getString(i-1,d,16);
    nAcc[i]=(unsigned char)MaxMin(64+accDiv*(int)(atof(d)*accMax/(float)accDiv));
    if(pAcc[i]!=nAcc[i]) {b=true;pAcc[i]=nAcc[i];}
  }
  if(b) for(i=0;i<4;i++) {Serial.write(nAcc[i]);}
}

void sensorGyroAttitude(OSCMessage &msg) {
  int i; b=false;
  for(i=1;i<5;i++) {
    msg.getString(i-1,d,16);
    nGyA[i]=(unsigned char)MaxMin(64+gyaDiv*(int)(atof(d)*gyaMax/(float)gyaDiv));
    if(pGyA[i]!=nGyA[i]) {b=true;pGyA[i]=nGyA[i];}
  }
  if(b) for(i=0;i<5;i++) { Serial.write(nGyA[i]); }
}

void sensorGyroUnbiased(OSCMessage &msg) {
  int i; b=false;
  for(i=1;i<4;i++) {
    msg.getString(i-1,d,16);
    nGyU[i]=(unsigned char)MaxMin(64+gyuDiv*(int)(atof(d)*gyuMax/(float)gyuDiv));
    if(pGyU[i]!=nGyU[i]) {b=true;pGyU[i]=nGyU[i];}
  }
  if(b) for(i=0;i<4;i++) {Serial.write(nGyU[i]);}
}

void sensorAudio(OSCMessage &msg) {
  msg.getString(0,d,16);
  nAudio=(unsigned char)MaxMin(audDiv*(int)(atof(d)*audMax/(float)audDiv));
  if(pAudio!=nAudio) {
    Serial.write(0x86);
    Serial.write(nAudio);
    pAudio=nAudio;
  }
}

/*********************************************
 * IP2String
 * IPアドレスを文字列にする
 * 
 ********************************************/
String IP2String(IPAddress address)
{
  return String(address[0]) + "." +
         String(address[1]) + "." +
         String(address[2]) + "." +
         String(address[3]);
}

/*********************************************
 * receiveHello
 * 接続
 * 
 *********************************************/

void receiveHello(OSCMessage &msg) {
  bConnect=true;
  
  char ip[16];

  //set outputIP from /hello message
  msg.getString(0, ip, 16);
  //Serial.print("'param 0 is "); Serial.println(ip);

  msg.getString(0, ip, 16);
  outIp.fromString(ip);
  //Serial.print("'change oututUP is "); Serial.println( IP2String(outIp) );

  //get 3rd param to check broadcast or not
  msg.getString(2, ip, 16);

  if (strlen(ip) == 0) {

    //this is braod cast
    //Serial.println("'no ips");

    //set aruduino ip
    IP2String(WiFi.localIP()).toCharArray(ip, 16);

    char port[6];
    sprintf(port, "%d", outPort);
    OSCMessage smsg("/config/network");
    smsg.add("hogehoge");  //this param is no use in current Unipad.
    smsg.add(ip);
    smsg.add(port);
    smsg.add(port);
    Udp.beginPacket(outIp, outPort);
    smsg.send(Udp);
    Udp.endPacket();
    smsg.empty();
    //Serial.printf("'set ip to %s:%d", ip, outPort);

  } else {
    //Serial.println("'set padtype");
    OSCMessage smsg("/config/type");
    smsg.add(padType);
    Udp.beginPacket(outIp, outPort);
    smsg.send(Udp);
    Udp.endPacket();
    smsg.empty();
  }

  char s1[50] = "<size=30>ESP IP is ";

  char s2[16];
  IP2String(WiFi.localIP()).toCharArray(s2, 16) ;
  char s3[9] = " </size>";
  strcat(s1, s2);
  strcat(s1, s3);

  OSCMessage imsg("/info");
  imsg.add( s1 );
  Udp.beginPacket(outIp, outPort);
  imsg.send(Udp);
  Udp.endPacket();
  imsg.empty();
  
}

/*********************************************
 * receiveBye
 * 切断
 * 
 *********************************************/
void receiveBye(OSCMessage &msg) {
  bConnect=false;
}

/*********************************************
 * ReadOSC
 * データチェック
 * 
 *********************************************/
void ReadOSC() {

  OSCMessage bundleIN;
  int rlen;
  
  if ( (rlen = Udp.parsePacket()) > 0)
  {
    //Serial.println(rlen);
    while (rlen--) {
      bundleIN.fill(Udp.read());
    }

    if (!bundleIN.hasError()) {

      bundleIN.dispatch("/touch/start", touchStart);
      bundleIN.dispatch("/touch/repeat", touchRepeat);
      bundleIN.dispatch("/touch/end", touchEnd);
      bundleIN.dispatch("/touch/pos", touchPos);
      bundleIN.dispatch("/touch/pos2", touchPos2);

      bundleIN.dispatch("/sensor/accel", sensorAccel);
      bundleIN.dispatch("/sensor/gyroAttitude", sensorGyroAttitude);
      bundleIN.dispatch("/sensor/gyroUnbiased", sensorGyroUnbiased);
      bundleIN.dispatch("/sensor/audio", sensorAudio);

      bundleIN.dispatch("/hello", receiveHello);
      bundleIN.dispatch("/bye", receiveBye);
      
    }
    else {
      error = bundleIN.getError();
      //Serial.print("'error: "); Serial.println(error);
    }
    
  }
}

