/*
 *  ESP_Unipad_Receiver by Michio Ono.
 *  ESP-WROOM-02 Unipad data reciever
 *  CC BY
 *  Ver 1.2.0
 *  
 *  latest version
 *  http://ijutilities.micutil.com
 * 
 */

 /******************************************
 * References
 * ****************************************
 * 
 * Unipad x ESP8266
 * http://unipad.digitalray.co.jp
 * 
 * tested on ESPr Developer
 * https://www.switch-science.com/catalog/2500/
 * 
 * IchigoJam
 * http://ichigojam.net
 * 
 */ 

#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

const char ssid[] = "****";  //  your network SSID (name)
const char pass[] = "****";  // your network password

WiFiUDP udp;
unsigned int localPort = 10000;
const int PACKET_SIZE = 256;
char packetBuffer[PACKET_SIZE];
bool bConnect=false;

//Buttons
unsigned char nBtn[3];
unsigned char pBtn[3];

//touchArea & tachArea1
unsigned char nPos[3],pPos[3];
int posDiv=4;//Sensitivity
float posMax=64.0/0.5-1.0;//Max value=0.5

//touchArea2
unsigned char nAnR[3],pAnR[3];
int anRDiv=4;//Sensitivity
float anRMax=64.0/0.5-1.0;//Max value=0.5

//acceleration sensor
unsigned char nAcc[4],pAcc[4];
int accDiv=4;//Sensitivity
float accMax=64.0/0.5-1.0;//Max value=5

//gyroscope attitude (姿勢）
unsigned char nGyA[5],pGyA[5];
int gyaDiv=4;//Sensitivity
float gyaMax=64.0/1.0-1.0;//Max value=1

//gyroscope Unbiased (回転角度）
unsigned char nGyU[4],pGyU[4];
int gyuDiv=4;//Sensitivity
float gyuMax=64.0/4.0-1.0;//Max value=4

//Audio
unsigned char nAudio,pAudio;
int audDiv=4;//Sensitivity
float audMax=128.0/10.0-1.0;//Max value=10

void setup() {
  Serial.begin(115200);
  while (!Serial) {
    ;// wait for serial port to connect. Needed for Leonardo only
  }

  WiFi.begin(ssid, pass);
  WiFi.mode(WIFI_STA);
  Serial.println("'Connecting to Access Point");
  Serial.print("'");//for IchigoJam
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  IPAddress myIP = WiFi.localIP();
  
  /*
  WiFi.softAP(ssid, pass);
  IPAddress myIP = WiFi.softAPIP();
  Serial.print("'AP IP address: ");
  */
  Serial.println(myIP);
  
  Serial.println("'Starting UDP");
  udp.begin(localPort);
  Serial.print("'Local port: ");
  Serial.println(udp.localPort());

  nBtn[0]=0x80;
  nPos[0]=0x81;
  nAnR[0]=0x82;
  nAcc[0]=0x83;
  nGyA[0]=0x84;
  nGyU[0]=0x85;
  //Audio=0x86
  
}

void loop() {
  ReadUDP();
  writeUDP();
  delay(10);
}

/*********************************************
 * ReadUDP
 * 
 * データ受診
 * 
 **********************************************/

void ReadUDP() {
  bool b=false;
  int i,rlen=0;
  char *pb;
  
  if ((rlen = udp.parsePacket())) {

    /*パケット読み込み*/
    udp.read(packetBuffer, (rlen > PACKET_SIZE) ? PACKET_SIZE : rlen);

    pb=packetBuffer;
    if(strncmp(pb,"/touch/start", 12)==0) {
      pb+=16; doTouchButton(pb,1);
      
    } else if(strncmp(pb,"/touch/repeat", 13)==0) {
      pb+=16; doTouchButton(pb,2);
      
    } else if(strncmp(pb,"/touch/end", 10)==0) {
      pb+=12; doTouchButton(pb,0);
      
    } else if(strncmp(pb,"/touch/pos2", 11)==0) {//こちらを先に！
      pb+=12; pb+=4;
      for(i=1;i<3;i++) {
        nAnR[i]=(unsigned char)MaxMin(64+anRDiv*(int)(atof(pb)*anRMax/(float)anRDiv));
        if(pAnR[i]!=nAnR[i]) {b=true;pAnR[i]=nAnR[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<3;i++) {Serial.write(nAnR[i]);}
      
    } else if(strncmp(pb,"/touch/pos", 10)==0) {//こちらを後に！
      pb+=12; pb+=4;
      for(i=1;i<3;i++) {
        nPos[i]=(unsigned char)MaxMin(64+posDiv*(int)(atof(pb)*posMax/(float)posDiv));
        if(pPos[i]!=nPos[i]) {b=true;pPos[i]=nPos[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<3;i++) {Serial.write(nPos[i]);}
      
    } else if(strncmp(pb,"/sensor/accel", 13)==0) {
      pb+=16;pb+=8;b=false;
      for(i=1;i<4;i++) {
        nAcc[i]=(unsigned char)MaxMin(64+accDiv*(int)(atof(pb)*accMax/(float)accDiv));
        if(pAcc[i]!=nAcc[i]) {b=true;pAcc[i]=nAcc[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<4;i++) {Serial.write(nAcc[i]);}
      
    } else if(strncmp(pb,"/sensor/gyroAttitude", 20)==0) {
      pb+=24;pb+=8;b=false;
      for(i=1;i<5;i++) {
        nGyA[i]=(unsigned char)MaxMin(64+gyaDiv*(int)(atof(pb)*gyaMax/(float)gyaDiv));
        if(pGyA[i]!=nGyA[i]) {b=true;pGyA[i]=nGyA[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<5;i++) { Serial.write(nGyA[i]); }
      
    } else if(strncmp(pb,"/sensor/gyroUnbiased", 20)==0) {
      pb+=24;pb+=8; b=false;
      for(i=1;i<4;i++) {
        nGyU[i]=(unsigned char)MaxMin(64+gyuDiv*(int)(atof(pb)*gyuMax/(float)gyuDiv));
        if(pGyU[i]!=nGyU[i]) {b=true;pGyU[i]=nGyU[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<4;i++) {Serial.write(nGyU[i]);}
      
    } else if(strncmp(pb,"/sensor/audio", 13)==0) {
      pb+=16;pb+=4;b=false;
      nAudio=(unsigned char)MaxMin(audDiv*(int)(atof(pb)*audMax/(float)audDiv));
      if(pAudio!=nAudio) {
        Serial.write(0x86);
        Serial.write(nAudio);
        pAudio=nAudio;
      }
      
    } else if(strncmp(pb,"/hello",6)==0) {
      bConnect=true;
    } else if(strncmp(pb,"/bye",4)==0) {
      bConnect=false;
    }
    /* else if(strncmp(pb,"/update",7)==0||
        strncmp(pb,"/test/alive/",12)==0) {
      ;
    }*/
  }
}

/*********************************************
 * doTouchButton
 * 
 * ボタンが押された時の処理 
 * 
 **********************************************/
void doTouchButton(char *pb, int rType) {
  int i,rlen=0, tch;
  if((tch=strlen(pb)-1)>0) {
    pb+=movePacketPos(pb);
    for(i=0; i<tch; i++) {
      if(strncmp(pb,"Up", 2)==0) { //1
        pb+=2;
        if(rType<2) {nBtn[2]&=0b11111110; nBtn[2]|=rType;}
      } else if(strncmp(pb,"Down", 4)==0) { //2
        pb+=4;
        if(rType<2) {nBtn[2]&=0b11111101; nBtn[2]|=(rType<<1);}
      } else if(strncmp(pb,"Right", 5)==0) { //4
        pb+=3;
        if(rType<2) {nBtn[2]&=0b11111011; nBtn[2]|=(rType<<2);}
      } else if(strncmp(pb,"Left", 4)==0) { //8
        pb+=4;
        if(rType<2) {nBtn[2]&=0b11110111; nBtn[2]|=(rType<<3);}
      } else if(strncmp(pb,"ButtonY", 7)==0) { //16
        pb+=1;
        if(rType<2) {nBtn[2]&=0b11101111; nBtn[2]|=(rType<<4);}
      } else if(strncmp(pb,"ButtonB", 7)==0) { //32
        pb+=1;
        if(rType<2) {nBtn[2]&=0b11011111; nBtn[2]|=(rType<<5);}
      } else if(strncmp(pb,"ButtonA", 7)==0) { //64
        pb+=1;
        if(rType<2) {nBtn[2]&=0b10111111; nBtn[2]|=(rType<<6);}
      } else if(strncmp(pb,"ButtonX", 7)==0) { //1
        pb+=1;
        if(rType<2) {nBtn[1]&=0b11111110; nBtn[1]|=rType;}
      } else if(strncmp(pb,"ButtonL1", 8)==0) { //2
        pb+=4;
        if(rType<2) {nBtn[1]&=0b11111101; nBtn[1]|=(rType<<1);}
      } else if(strncmp(pb,"ButtonL2", 8)==0) { //3
        pb+=4;
        if(rType<2) {nBtn[1]&=0b11111011; nBtn[1]|=(rType<<2);}
      } else if(strncmp(pb,"ButtonR1", 8)==0) { //4
        pb+=4;
        if(rType<2) {nBtn[1]&=0b11110111; nBtn[1]|=(rType<<3);}
      } else if(strncmp(pb,"ButtonR2", 8)==0) { //5
        pb+=4;
        if(rType<2) {nBtn[1]&=0b11101111; nBtn[1]|=(rType<<4);}
      } else if(strncmp(pb,"ButtonS1", 8)==0) { //3
        pb+=4;
        if(rType<2) {nBtn[2]&=0b11111100; nBtn[2]|=(0x03);}
      } else if(strncmp(pb,"ButtonS2", 8)==0) { //12
        pb+=4;
        if(rType<2) {nBtn[2]&=0b11110011; nBtn[2]|=(0x0C);}
      } else if(rType==0&&strncmp(pb,"TouchArea", 9)==0) {
        pb+=3; nPos[1]=nPos[2]=pPos[1]=pPos[2]=64;
        for(i=0;i<3;i++) {Serial.write(nPos[i]);}
        return;
      } else if(rType==0&&strncmp(pb,"TouchArea2", 10)==0) {
        pb+=3; nAnR[1]=nAnR[2]=pAnR[1]=pAnR[2]=64;
        for(i=0;i<3;i++) {Serial.write(nAnR[i]);}
        return;
      } else if(strncmp(pb,"Menu",4)==0) {
        return;
      }
    }

    //状態を送信
    if( (pBtn[1]!=nBtn[1]) || (pBtn[2]!=nBtn[2]) ) {
      Serial.write(nBtn[0]);Serial.write(nBtn[1]); Serial.write(nBtn[2]);
      pBtn[1]=nBtn[1];pBtn[2]=nBtn[2];
    }

  } else {
    //解放状態
    nBtn[1]=pBtn[1]=nBtn[2]=pBtn[2]=0;
    Serial.write(nBtn[0]);Serial.write(nBtn[1]); Serial.write(nBtn[2]);
  }
  
}

/*********************************************
 * moobePacketPos
 * 
 * 次のコマンドに進む
 * 4byte単位で空読みされてる部分を移動
 * 
 **********************************************/
int movePacketPos(char *pb) {
  int n=strlen(pb);
  if(n<4) {
    return 4;
  } else if(n<8) {
    return 8;
  } else if(n<12) {
    return 12;
  }
  return 0;
}

/*********************************************
 * 最大値・最小値
 * 
 * 1-127: center 64, from -63 to 63 
 * 
 **********************************************/

inline int max(int a,int b) {return ((a)>(b)?(a):(b)); }
inline int min(int a,int b) {return ((a)<(b)?(a):(b)); }
int MaxMin(int n) {
  return min(max(1,n),127);
}

/*********************************************
 * writeUDP
 * 
 * データ送信
 * 
 **********************************************/

void writeUDP() {
  //テスト中
  
  if(bConnect=false) return;

  String inStr = ""; // a string to hold incoming data
  boolean stringComplete = false;  // whether the string is complete
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read();
    //Check end of line 
    if(inChar=='\n' || inChar=='\r') {
      stringComplete = true;
      break;
    } else {
      inStr += inChar;
    }
  }

  if(stringComplete) {  //データあり
    stringComplete=false;
    if (udp.beginPacket(udp.remoteIP(), udp.remotePort())) {
      if(inStr.indexOf("/vib")>=0) {
        udp.write("/vib");
        udp.endPacket();
      }
    }
  }
  
}

