/*
 *  ESP_Unipad_Receiver by Michio Ono.
 *  ESP-WROOM-02 Unipad data reciever
 *  Websocket Edition
 *  CC BY
 *  Ver 1.1.0
 *  
 *  latest version
 *  http://ijutilities.micutil.com
 *  
 */

  /******************************************
 * References
 * ****************************************
 * 
 * Unipad x ESP8266
 * http://unipad.digitalray.co.jp
 * 
 * tested on ESPr Developer
 * https://www.switch-science.com/catalog/2500/
 * 
 * IchigoJam
 * http://ichigojam.net
 * 
 */ 

 #include <Arduino.h>

#include <ESP8266WiFi.h>
//#include <ESP8266WiFiMulti.h>
#include <WebSocketsServer.h>
#include <Hash.h>

//ESP8266WiFiMulti WiFiMulti;

WebSocketsServer webSocket = WebSocketsServer(81);
void ReadWS(uint8_t * payload, size_t lenght);
void writeWS();
uint8_t WS_num = 0;
const char ssid[] = "****";  //  your network SSID (name)
const char pass[] = "****";  // your network password
bool bConnect=false;

//Buttons
unsigned char nBtn[3];
unsigned char pBtn[3];

//touchArea & tachArea1
unsigned char nPos[3],pPos[3];
int posDiv=4;//Sensitivity
float posMax=64.0/0.5-1.0;//Max value=0.5

//touchArea2
unsigned char nAnR[3],pAnR[3];
int anRDiv=4;//Sensitivity
float anRMax=64.0/0.5-1.0;//Max value=0.5

//acceleration sensor
unsigned char nAcc[4],pAcc[4];
int accDiv=4;//Sensitivity
float accMax=64.0/0.5-1.0;//Max value=5

//gyroscope attitude (姿勢）
unsigned char nGyA[5],pGyA[5];
int gyaDiv=4;//Sensitivity
float gyaMax=64.0/1.0-1.0;//Max value=1

//gyroscope Unbiased (回転角度）
unsigned char nGyU[4],pGyU[4];
int gyuDiv=4;//Sensitivity
float gyuMax=64.0/4.0-1.0;//Max value=4

//Audio
unsigned char nAudio,pAudio;
int audDiv=4;//Sensitivity
float audMax=128.0/10.0-1.0;//Max value=10

void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t lenght) {

    switch(type) {
        case WStype_DISCONNECTED:
            bConnect=false;
//            Serial.printf("\n[%u] Disconnected!\n", num);
            break;
        case WStype_CONNECTED:
            {
                bConnect=true;
                WS_num = num;
                IPAddress ip = webSocket.remoteIP(num);
//                Serial.printf("\n[%u] Connected from %d.%d.%d.%d url: %s\n", num, ip[0], ip[1], ip[2], ip[3], payload);
                // send message to client
                webSocket.sendTXT(num, "/config/type 0");
            }
            break;
        case WStype_TEXT:
//            Serial.printf("[%u] get Text: %s\n", num, payload);
            ReadWS(payload,lenght);
            // send message to client
            // webSocket.sendTXT(num, "message here");
            // send data to all connected clients
            // webSocket.broadcastTXT("message here");
            break;
        case WStype_BIN:
//            Serial.printf("[%u] get binary lenght: %u\n", num, lenght);
            hexdump(payload, lenght);

            // send message to client
            // webSocket.sendBIN(num, payload, lenght);
            break;
    }
}

void setup() {
  Serial.begin(115200);
  while (!Serial) {
    ;// wait for serial port to connect. Needed for Leonardo only
  }
    //Serial.setDebugOutput(false);
    //Serial.setDebugOutput(true);
    /*
    Serial.println();
    Serial.println();
    Serial.println();

    for(uint8_t t = 4; t > 0; t--) {
        Serial.printf("'[SETUP] BOOT WAIT %d...\n", t);
        Serial.flush();
        delay(1000);
    }

    WiFiMulti.addAP(ssid, pass);
    Serial.print("'");
    while(WiFiMulti.run() != WL_CONNECTED) {
      Serial.print(".");
        delay(100);
    }
    */

  WiFi.begin(ssid, pass);
  WiFi.mode(WIFI_STA);
  Serial.println("'Connecting to Access Point");
  Serial.print("'");//for IchigoJam
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  IPAddress myIP = WiFi.localIP();
  
  /*
  WiFi.softAP(ssid, pass);
  IPAddress myIP = WiFi.softAPIP();
  Serial.print("'AP IP address: ");
  */
  Serial.println(myIP);

  webSocket.begin();
  webSocket.onEvent(webSocketEvent);

  nBtn[0]=0x80;
  nPos[0]=0x81;
  nAnR[0]=0x82;
  nAcc[0]=0x83;
  nGyA[0]=0x84;
  nGyU[0]=0x85;
  //Audio=0x86
  
}

void loop() {
  webSocket.loop();
  writeWS();
  delay(10);
}

/*********************************************
 * replacePacketZero
 * 
  *********************************************/
void replacePacketZero(char *pb, int n) {
  for(int i=0; i<n; i++) {
     if(pb[i]=='"'||pb[i]=='['||pb[i]==']'||pb[i]==',') pb[i]=0x0;
  }
}
  
/*********************************************
 * moobePacketPos
 * 
 * 次のコマンドに進む
 * 4byte単位で空読みされてる部分を移動
 * 
 **********************************************/
int movePacketPos(char *pb) {
  int n=strlen(pb);
  return n+3;
}

/*********************************************
 * doTouchButton
 * 
 * ボタンが押された時の処理 
 * 
 **********************************************/
void doTouchButton(char *pb, int rType) {
  int i;//,rlen=0, tch;
  if(strlen(pb)>0) {
    while(strlen(pb)>0) {
      if(strncmp(pb,"Up", 2)==0) { //1
        if(rType<2) {nBtn[2]&=0b11111110; nBtn[2]|=rType;}
      } else if(strncmp(pb,"Down", 4)==0) { //2
        if(rType<2) {nBtn[2]&=0b11111101; nBtn[2]|=(rType<<1);}
      } else if(strncmp(pb,"Right", 5)==0) { //4
        if(rType<2) {nBtn[2]&=0b11111011; nBtn[2]|=(rType<<2);}
      } else if(strncmp(pb,"Left", 4)==0) { //8
        if(rType<2) {nBtn[2]&=0b11110111; nBtn[2]|=(rType<<3);}
      } else if(strncmp(pb,"ButtonY", 7)==0) { //16
        if(rType<2) {nBtn[2]&=0b11101111; nBtn[2]|=(rType<<4);}
      } else if(strncmp(pb,"ButtonB", 7)==0) { //32
        if(rType<2) {nBtn[2]&=0b11011111; nBtn[2]|=(rType<<5);}
      } else if(strncmp(pb,"ButtonA", 7)==0) { //64
        if(rType<2) {nBtn[2]&=0b10111111; nBtn[2]|=(rType<<6);}
      } else if(strncmp(pb,"ButtonX", 7)==0) { //1
        if(rType<2) {nBtn[1]&=0b11111110; nBtn[1]|=rType;}
      } else if(strncmp(pb,"ButtonL1", 8)==0) { //2
        if(rType<2) {nBtn[1]&=0b11111101; nBtn[1]|=(rType<<1);}
      } else if(strncmp(pb,"ButtonL2", 8)==0) { //3
        if(rType<2) {nBtn[1]&=0b11111011; nBtn[1]|=(rType<<2);}
      } else if(strncmp(pb,"ButtonR1", 8)==0) { //4
        if(rType<2) {nBtn[1]&=0b11110111; nBtn[1]|=(rType<<3);}
      } else if(strncmp(pb,"ButtonR2", 8)==0) { //5
         if(rType<2) {nBtn[1]&=0b11101111; nBtn[1]|=(rType<<4);}
      } else if(strncmp(pb,"ButtonS1", 8)==0) { //3
        if(rType<2) {nBtn[2]&=0b11111100; nBtn[2]|=(0x03);}
      } else if(strncmp(pb,"ButtonS2", 8)==0) { //12
        if(rType<2) {nBtn[2]&=0b11110011; nBtn[2]|=(0x0C);}
      } else if(rType==0&&strncmp(pb,"TouchArea", 9)==0) {
        nPos[1]=nPos[2]=pPos[1]=pPos[2]=64;
        for(i=0;i<3;i++) {Serial.write(nPos[i]);}
        return;
      } else if(rType==0&&strncmp(pb,"TouchArea2", 10)==0) {
        nAnR[1]=nAnR[2]=pAnR[1]=pAnR[2]=64;
        for(i=0;i<3;i++) {Serial.write(nAnR[i]);}
        return;
      } else if(strncmp(pb,"Menu",4)==0) {
        return;
      }
      pb+=movePacketPos(pb);
    }

    //状態を送信
    if( (pBtn[1]!=nBtn[1]) || (pBtn[2]!=nBtn[2]) ) {
      Serial.write(nBtn[0]);Serial.write(nBtn[1]); Serial.write(nBtn[2]);
      pBtn[1]=nBtn[1];pBtn[2]=nBtn[2];
    }

  } else {
    //解放状態
    nBtn[1]=pBtn[1]=nBtn[2]=pBtn[2]=0;
    Serial.write(nBtn[0]);Serial.write(nBtn[1]); Serial.write(nBtn[2]);    
  }
  
}

/*********************************************
 * 最大値・最小値
 * 
 * 1-127: center 64, from -63 to 63 
 * 
 **********************************************/

inline int max(int a,int b) {return ((a)>(b)?(a):(b)); }
inline int min(int a,int b) {return ((a)<(b)?(a):(b)); }
int MaxMin(int n) {
  return min(max(1,n),127);
}

/*********************************************
 * ReadWS
 * 
 * データ受診
 * 
 **********************************************/

void ReadWS(uint8_t * payload, size_t lenght) {
  bool b=false;
  int i,rlen=0;
  char *pb;
  
  if (lenght > 0) {

    /*パケット読み込み*/
    pb=(char*)payload;
    replacePacketZero(pb,lenght);
    //Serial.println(pb); return;
    
    if(strncmp(pb,"/touch/start", 12)==0) {
      pb+=12;pb+=2;doTouchButton(pb,1);
      
    } else if(strncmp(pb,"/touch/repeat", 13)==0) {
      pb+=13;pb+=3;doTouchButton(pb,2);
      
    } else if(strncmp(pb,"/touch/end", 10)==0) {
      pb+=10;pb+=2;doTouchButton(pb,0);
      
    } else if(strncmp(pb,"/touch/pos2", 11)==0) {//こちらを先に！
      pb+=12; pb+=4;b=false;
      for(i=1;i<3;i++) {
        nAnR[i]=(unsigned char)MaxMin(64+anRDiv*(int)(atof(pb)*anRMax/(float)anRDiv));
        if(pAnR[i]!=nAnR[i]) {b=true;pAnR[i]=nAnR[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<3;i++) {Serial.write(nAnR[i]);}
      
    } else if(strncmp(pb,"/touch/pos", 10)==0) {//こちらを後に！
      pb+=10;pb+=3;b=false;
      for(i=1;i<3;i++) {
        nPos[i]=(unsigned char)MaxMin(64+posDiv*(int)(atof(pb)*posMax/(float)posDiv));
        if(pPos[i]!=nPos[i]) {b=true;pPos[i]=nPos[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<3;i++) {Serial.write(nPos[i]);}
      
    } else if(strncmp(pb,"/sensor/accel", 13)==0) {
      pb+=13;pb+=3;b=false;
      for(i=1;i<4;i++) {
        nAcc[i]=(unsigned char)MaxMin(64+accDiv*(int)(atof(pb)*accMax/(float)accDiv));
        if(pAcc[i]!=nAcc[i]) {b=true;pAcc[i]=nAcc[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<4;i++) {Serial.write(nAcc[i]);}
      
    } else if(strncmp(pb,"/sensor/gyroAttitude", 20)==0) {
      pb+=20;pb+=3;b=false;
      for(i=1;i<5;i++) {
        nGyA[i]=(unsigned char)MaxMin(64+gyaDiv*(int)(atof(pb)*gyaMax/(float)gyaDiv));
        if(pGyA[i]!=nGyA[i]) {b=true;pGyA[i]=nGyA[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<5;i++) { Serial.write(nGyA[i]); }
      
    } else if(strncmp(pb,"/sensor/gyroUnbiased", 20)==0) {
      pb+=20;pb+=3;b=false;
      for(i=1;i<4;i++) {
        nGyU[i]=(unsigned char)MaxMin(64+gyuDiv*(int)(atof(pb)*gyuMax/(float)gyuDiv));
        if(pGyU[i]!=nGyU[i]) {b=true;pGyU[i]=nGyU[i];}
        pb+=movePacketPos(pb);
      }
      if(b) for(i=0;i<4;i++) {Serial.write(nGyU[i]);}
      
    } else if(strncmp(pb,"/sensor/audio", 13)==0) {
      pb+=13;pb+=3; b=false;
      nAudio=(unsigned char)MaxMin(audDiv*(int)(atof(pb)*audMax/(float)audDiv));
      if(pAudio!=nAudio) {
        Serial.write(0x86);
        Serial.write(nAudio);
        pAudio=nAudio;
      }
      
    } else if(strncmp(pb,"/hello",6)==0) {
      bConnect=true;
//      webSocket.sendTXT(WS_num, "/config/type 0");
    } else if(strncmp(pb,"/bye",4)==0) {
      bConnect=false;
    }
    /* else if(strncmp(pb,"/update",7)==0||
        strncmp(pb,"/test/alive/",12)==0) {
      ;
    }*/
  }
}

/*********************************************
 * writeWS
 * 
 * データ送信
 * 
 **********************************************/

void writeWS() {
  //テスト中
  if(bConnect==false) return;

  String inStr = ""; // a string to hold incoming data
  boolean stringComplete = false;  // whether the string is complete
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read();
    //Check end of line 
    if(inChar=='\n' || inChar=='\r') {
      stringComplete = true;
      break;
    } else {
      inStr += inChar;
    }
  }

  if(stringComplete) {  //データあり
    stringComplete=false;
//      webSocket.sendTXT(WS_num, "/config/type 1");
      if(inStr.indexOf("/vib")>=0) {
          webSocket.sendTXT(WS_num, "/vib");
      }
  }
  
}

